package nazou.support.jobofferportal.repository;

import nazou.support.jobofferportal.model.*;
import nazou.support.jobofferportal.graphmodel.*;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import org.openrdf.model.*;
import org.openrdf.model.impl.LiteralImpl;
import org.openrdf.model.impl.StatementImpl;
import org.openrdf.model.impl.URIImpl;
import org.openrdf.sesame.sail.StatementIterator;
import org.openrdf.sesame.config.AccessDeniedException;
import org.openrdf.sesame.constants.QueryLanguage;
import org.openrdf.sesame.query.MalformedQueryException;
import org.openrdf.sesame.query.QueryEvaluationException;
import org.openrdf.sesame.query.QueryResultsTable;
import org.openrdf.sesame.repository.*;
import org.openrdf.vocabulary.RDFS;

/**
 * 
 * Class for Bean to Graph, and Graph to Bean transformations
 * 
 * @author Peter Bartalos
 *
 */
public class BeanGraphTransformer {

	private String metadataFileName = null;
	
	public String getMetadataFileName() {
		return metadataFileName;
	}

	public void setMetadataFileName(String metadataFileName) {
		this.metadataFileName = metadataFileName;
	}

	public BeanGraphTransformer(String metadataFileName) {
		super();
		setMetadataFileName(metadataFileName);
	}

	/**
	 * 
	 * N 25.3.2006, PB 
	 * C 8.4.2006, PB: added type predicate value setting (resolved the problem with multiple type statements, example - benefit type)
	 * Fills the bean fields from information from the graph (recursively creates instances of required
	 * beans and fils them from graphs obtained from the repository)
	 * 
	 * @param graph
	 * @param bean which fields we want to fill
	 * @param prefix of the property in XML (for example "salary" for the bonus) 
	 */
	public void transformGraphToBean(Graph graph, BeanParent bean, String propertyPrefix) throws Exception{
		
		if(graph == null || bean == null || propertyPrefix == null){
			throw new NullPointerException();
		}
		
		try{
		GraphBeanTransformerMetadataXMLReader reader = new GraphBeanTransformerMetadataXMLReader(getMetadataFileName());
		StatementIterator iterator;
		Statement statement = null;
		OfferFieldMetadataObject metaObject = null;
		iterator = graph.getStatements();
		SesameRepositoryAccess repositoryAccess= new SesameRepositoryAccess(getMetadataFileName());


		String fieldName = null;
		
		while(iterator.hasNext()){
			statement = iterator.next();
//			System.out.println("statement: " + statement);
			URI subject = (URI) statement.getSubject();
			bean.setID(subject.toString());
			URI predicate = (URI) statement.getPredicate();
			metaObject = reader.getMetadataObject(predicate.toString(), propertyPrefix);
			if(metaObject == null) continue;
			fieldName = cutPrefix(propertyPrefix, metaObject.getBeanFieldName());
			
			if(metaObject.getMultiple().compareTo("true") == 0){
				if(metaObject.getType().compareTo("object") != 0){
					

					List list = (List) getFieldValue(bean, fieldName);
					
					if(metaObject.getType().compareTo("objectType") == 0){
						
						URI object = null;
						if(metaObject.getOntologyPropertyURI().compareTo("http://www.w3.org/1999/02/22-rdf-syntax-ns#type") == 0){
							object = new URIImpl(getTypeValue(repositoryAccess, bean.getID()));
						}
						else{
							object = (URI) statement.getObject();
						}

						list.add(object.toString());
						continue;
					}
					
					Literal object = (Literal) statement.getObject();
					
					if(metaObject.getType().compareTo("date") == 0){
						Date date = getDate(object.toString());
						list.add(date);
						continue;
					}

					if(metaObject.getType().compareTo("float") == 0){
						list.add(Float.parseFloat(object.toString()));
						continue;
					}
					
					if(metaObject.getType().compareTo("integer") == 0){
						list.add(Integer.parseInt(object.toString()));
						continue;
					}
					
					if(metaObject.getType().compareTo("boolean") == 0){
						list.add(new Boolean(object.toString()));
						continue;
					}
					list.add(object.toString());
				}
				URI object = (URI) statement.getObject();
				
				List list = (List) getFieldValue(bean, fieldName);
				
				Class c = null;
				BeanParent instance = null;
				c = Class.forName("nazou.support.jobofferportal.model." + metaObject.getBeanName());
				instance = (BeanParent) c.newInstance();

				Graph instanceGraph = repositoryAccess.getOfferGraphFromRepository(object.toString());
				transformGraphToBean(instanceGraph, instance, propertyPrefix + "." + fieldName);
				list.add(instance);
				
				
			}
			else if(metaObject.getType().compareTo("object") == 0){
				URI object = (URI) statement.getObject();
				
				BeanParent instance = createInstanceOfField(fieldName, bean);
				instance.setID(object.toString());
				setField(bean, fieldName, instance);

				Graph instanceGraph = repositoryAccess.getOfferGraphFromRepository(object.toString());
				transformGraphToBean(instanceGraph, instance, propertyPrefix + "." + fieldName);
			}
			else if(metaObject.getType().compareTo("objectType") == 0){
				URI object = null;
				if(metaObject.getOntologyPropertyURI().compareTo("http://www.w3.org/1999/02/22-rdf-syntax-ns#type") == 0){
					object = new URIImpl(getTypeValue(repositoryAccess, bean.getID()));
				}
				else{
					object = (URI) statement.getObject();
				}
				
				setField(bean, fieldName, object.toString());
			}
			else{
				Literal object = (Literal) statement.getObject();
				
				if(metaObject.getType().compareTo("date") == 0){
					Date date = getDate(object.toString());
					setField(bean, fieldName, date);
					continue;
				}
				
				if(metaObject.getType().compareTo("float") == 0){
					setField(bean, fieldName, Float.parseFloat(object.toString()));
					continue;
				}

				if(metaObject.getType().compareTo("integer") == 0){
					setField(bean, fieldName, Integer.parseInt(object.toString()));
					continue;
				}
				
				if(metaObject.getType().compareTo("boolean") == 0){
					setField(bean, fieldName, new Boolean(object.toString()));
					continue;
				}
				setField(bean, fieldName, object.toString());
				
			}
			



		}
		}
		catch (Exception e){
			e.printStackTrace();
			throw e;
		}

	}
	
	/**
	 * 
	 * N 8.4.2006, PB
	 * 
	 * Get the type of the instance defined by subjectID (it performs a query to repository)
	 * 
	 * @param repositoryAccess
	 * @param subjectID
	 * @return
	 */
	private String getTypeValue(SesameRepositoryAccess repositoryAccess, String subjectID){
		String typeValue = null;
		String query = "SELECT type FROM {<" + subjectID + ">} serql:directType {type}";
		try{
			QueryResultsTable resultsTable = repositoryAccess.getRepository().performTableQuery(QueryLanguage.SERQL, query);
			int rowCount = resultsTable.getRowCount();
			int columnCount = resultsTable.getColumnCount();
			for (int row = 0; row < rowCount; row++) {
			    for (int column = 0; column < columnCount; column++) {
			    	Value value = resultsTable.getValue(row, column);
			    	if (value != null) {
			    		if(value.toString().substring(0, 4).compareTo("node") != 0){
			    			typeValue = value.toString();
			    			break;
			    		}
			    	}
			    }
			}

		}
		catch (Exception e){
			e.printStackTrace();
		}
		
		return typeValue;
	}
	
	private Date getDate(String dateString){
		String[] s = dateString.split("-");
		
		if(s.length != 3){
			return null;
		}
		Date date = new Date(Integer.valueOf(s[0]).intValue() - 1900, Integer.valueOf(s[1]).intValue() - 1, Integer.valueOf(s[2]).intValue());
		
		return date;
	}
	
	
	/**
	 * N 27.3.2006, PB
	 * 
	 * @param fieldName the name of the field which is an object which instance we want to create
	 * @param bean beadn containing the field fieldName
	 * @return new instance of a class of the field fieldName
	 */
	private BeanParent createInstanceOfField(String fieldName, BeanParent bean) {
		BeanParent instance = null;
		try{
			Field[] fields = getFields(bean.getClass().getName());
		
			for(int i = 0; i < fields.length; i++){
				if(getAfterLastDotString(fields[i].getName()).compareTo(fieldName) == 0){
					instance = (BeanParent) fields[i].getType().newInstance();
					
				}
			}
			return instance;
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return instance;
	}

	/**
	 * N 27.3.2006, PB
	 * 
	 * Sets the field fieldName in a object to the value
	 * 
	 * @param object
	 * @param fieldName
	 * @param value
	 */
	private void setField(Object object, String fieldName, Object value){
		try{

			Class c = object.getClass();
			Method m = c.getMethod( toSetterName(fieldName), new Class[] {value.getClass()} );
			m.invoke( object, new Object[] {value} );
			
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * N 8.4.2006, PB
	 * 
	 * Gets the field fieldName value
	 * 
	 * @param object
	 * @param fieldName
	 * @param value
	 */
	private Object getFieldValue(Object object, String fieldName){
		Object value = null;
		try{
			Class c = object.getClass();
			Method m = c.getMethod( toGetterName(fieldName), null );
			value = m.invoke( object, null );
			return value;
			
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return value;
	}
	
	private String toSetterName(String name)
	{
		StringBuffer sb = new StringBuffer();
		sb.append( "set" ).append( name.substring(0,1).toUpperCase() ).append( name.substring(1) );
		return sb.toString();
	}
	
	private String toGetterName(String name)
	{
		StringBuffer sb = new StringBuffer();
		sb.append( "get" ).append( name.substring(0,1).toUpperCase() ).append( name.substring(1) );
		return sb.toString();
	}
	
	
	private String getAfterLastDotString(String string){
		String[] s = string.replace('.','-').split("-");
		return s[s.length-1];
	}
	
	/**
	 * N 25.3.2006, PB
	 * 
	 * @param prefix
	 * @param string
	 * @return end part (without dot) of string if prefix is a maximum prefix of string (i.e. the end part is of form: ".*"), else return false
	 */
	private String cutPrefix(String prefix, String string){
		String result = null;
		if(!string.startsWith(prefix)){
			return null;
		}
		result = string.substring(prefix.length()+1, string.length());
		
		return result;
	}
	
	/**
	 * N 18.3.2006, PB
	 * C 25.3.2006, PB: added objectType case, refactoring
	 * Transforms the bean to list of graphs like its equivalent
	 * 
	 * @param bean which we want to transform
	 * @param graphList list of graphs where the bean graph equivalents will be added
	 * @param prefix of the property in XML (for example "salary" for the bonus)
	 */
	public void transformBeanToGraph(BeanParent bean, List<Graph> graphList, String propertyPrefix) throws Exception {
		
		try{
		
		GraphBeanTransformerMetadataXMLReader reader = new GraphBeanTransformerMetadataXMLReader(getMetadataFileName());
		Graph graph = new org.openrdf.model.impl.GraphImpl();
		Field[] fields = getFields(bean.getClass().getName());
		String[] fieldNames = getFieldNames(bean.getClass().getName());
		OfferFieldMetadataObject metaObject = null;
		
			for(int i = 0; i < fieldNames.length; i++){
				if(getFieldValue(bean, fieldNames[i]) == null) continue;
				
				if(getFieldValue(bean, fieldNames[i]).toString().compareTo("")==0) continue;
				
				if(fields[i].getType().toString().compareTo("interface java.util.List") == 0){
					List list = (List) getFieldValue(bean, fieldNames[i]);
					
					metaObject = reader.getPropertyValue(propertyPrefix + "." + fieldNames[i]);
					for(int j = 0; j < list.size(); j++){
						if(list.get(j) == null){
							continue;
						}

						if(metaObject.getType().compareTo("object") != 0 && metaObject.getType().compareTo("objectType") != 0){
							if(metaObject.getType().compareTo("string") == 0){
								Statement statement = createDatatypeStatement(bean, (String) list.get(j), metaObject, org.openrdf.vocabulary.XmlSchema.STRING);
								graph.add(statement);
							}
							else if(metaObject.getType().compareTo("int") == 0){
								Statement statement = createDatatypeStatement(bean, (String) list.get(j), metaObject, org.openrdf.vocabulary.XmlSchema.INT);
								graph.add(statement);
							}
							else if(metaObject.getType().compareTo("float") == 0){
								Statement statement = createDatatypeStatement(bean, (String) list.get(j), metaObject, org.openrdf.vocabulary.XmlSchema.FLOAT);
								graph.add(statement);
							}
							else if(metaObject.getType().compareTo("boolean") == 0){
								Statement statement = createDatatypeStatement(bean, (String) list.get(j), metaObject, org.openrdf.vocabulary.XmlSchema.BOOLEAN);
								graph.add(statement);
							}
							else if(metaObject.getType().compareTo("label") == 0){
//								Statement statement = createDatatypeStatement(bean, (String) list.get(j), metaObject, RDFS.LABEL);
//								graph.add(statement);
							}

						}
						else if(metaObject.getType().compareTo("objectType") == 0){
							Statement statement = createObjecttypeStatement(bean, metaObject, (String) list.get(j));
							graph.add(statement);
							
						}
						else{
							if(((BeanParent) list.get(j)).getID() == null){
								continue;
							}
							Statement statement = createObjecttypeStatement(bean, metaObject, ((BeanParent) list.get(j)).getID());
							graph.add(statement);
							transformBeanToGraph((BeanParent) list.get(j),  graphList, propertyPrefix + "." + fieldNames[i]);
						}
					}
					continue;
				}
				metaObject = reader.getPropertyValue(propertyPrefix + "." + fieldNames[i]);
				if(metaObject == null) continue;
				if(metaObject.getType().compareTo("string") == 0){
					Statement statement = createDatatypeStatement(bean, getFieldValue(bean, fieldNames[i]).toString(), metaObject, org.openrdf.vocabulary.XmlSchema.STRING);
					graph.add(statement);
				}
				else if(metaObject.getType().compareTo("int") == 0){
					Statement statement = createDatatypeStatement(bean, getFieldValue(bean, fieldNames[i]).toString(), metaObject, org.openrdf.vocabulary.XmlSchema.INT);
					graph.add(statement);
				}
				else if(metaObject.getType().compareTo("float") == 0){
					Statement statement = createDatatypeStatement(bean, getFieldValue(bean, fieldNames[i]).toString(), metaObject, org.openrdf.vocabulary.XmlSchema.FLOAT);
					graph.add(statement);
				}
				else if(metaObject.getType().compareTo("boolean") == 0){
					Statement statement = createDatatypeStatement(bean, getFieldValue(bean, fieldNames[i]).toString(), metaObject, org.openrdf.vocabulary.XmlSchema.BOOLEAN);
					graph.add(statement);
				}
				else if(metaObject.getType().compareTo("label") == 0){
//					Statement statement = createDatatypeStatement(bean, getFieldValue(bean, fieldNames[i]).toString(), metaObject, RDFS.LABEL);
//					graph.add(statement);
				}
				else if(metaObject.getType().compareTo("date") == 0){
					String literal = formatDate((Date) getFieldValue(bean, fieldNames[i]));
 
					Statement statement = createDatatypeStatement(bean, literal, metaObject, org.openrdf.vocabulary.XmlSchema.DATE);
					graph.add(statement);
				}
				else if(metaObject.getType().compareTo("objectType") == 0){
					Statement statement = createObjecttypeStatement(bean, metaObject, getFieldValue(bean, fieldNames[i]).toString());
					graph.add(statement);
				}

				else {
					Statement statement = createObjecttypeStatement(bean, metaObject, ((BeanParent) getFieldValue(bean, fieldNames[i])).getID());
					graph.add(statement);
					if(metaObject.getRecursive().compareTo("true") == 0){
						transformBeanToGraph((BeanParent) getFieldValue(bean, fieldNames[i]),  graphList, propertyPrefix + "." + fieldNames[i]);
					}
				}

			}
			graphList.add(graph);
		}
		catch(Exception e){
			e.printStackTrace();
			throw e;
		}

		
	}


	
	private String formatDate(Date date){
//		DateFormat f = DateFormat.getDateInstance();
		DateFormat f = new SimpleDateFormat( "yyyy-MM-dd" );
		String s = f.format(date);
//		s = s.replace('.', '-');
//		String[] s1 = s.split("-");
//		return s1[2]+ "-" + s1[1] + "-" + s1[0];
		return s;
	}
	

	private Statement createObjecttypeStatement(BeanParent bean, OfferFieldMetadataObject metaObject, String objectValue) throws IllegalAccessException {
		URI subject = new URIImpl(bean.getID());
		URI predicate = new URIImpl(metaObject.getOntologyPropertyURI());
		URI object = new URIImpl(objectValue);

		Statement statement = new StatementImpl(subject, predicate,  object);
		return statement;
	}

	private Statement createDatatypeStatement(BeanParent bean, String literal, OfferFieldMetadataObject metaObject, String literalType) throws IllegalAccessException {
		URI subject = new URIImpl(bean.getID());
		URI predicate = new URIImpl(metaObject.getOntologyPropertyURI());
		Literal object = new LiteralImpl(literal, new URIImpl(literalType));
		
		
		Statement statement = new StatementImpl(subject, predicate,  object);

		return statement;
	}

	private String[] getFieldNames(String className) {
		String[] fieldNames = null;
		try{
			Class c = Class.forName(className);
			Field fields[] = c.getDeclaredFields();
			fieldNames = new String[fields.length];
			
			for(int i = 0; i < fieldNames.length; i++){
				// the split is crazy!!!
				String str = fields[i].toString().replace('.', '-');
				String[] s = str.split("-");
				
				fieldNames[i] = s[s.length - 1];
			}
			
		}
		catch(ClassNotFoundException e){
			e.printStackTrace();
		}
		return fieldNames;
	}
	
	private Field[] getFields(String className) {
		Field fields[] = null;
		try{
			Class c = Class.forName(className);
			fields = c.getDeclaredFields();
		}
		catch(ClassNotFoundException e){
			e.printStackTrace();
		}
		return fields;
	}
	

	protected String cutNamespace(String statement){
		String[] s= statement.split("#");
		return s[1];
	}
	
}

