package nazou.support.jobofferportal.repository;

import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
 
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;  

import java.io.File;
import java.io.IOException;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.*;

import java.util.*;

/**
 * Class serves as a graph-bean transformer metadata XML file reader
 * 
 * @author Peter Bartalos
 *
 */
public class GraphBeanTransformerMetadataXMLReader {

	Hashtable<String, OfferFieldMetadataObject> mappingRuleNodes = null;
	String fileName = null;

	public GraphBeanTransformerMetadataXMLReader(String _fileName) {
//		System.out.println("G2B_DEBUG:" + _fileName);
		fileName = _fileName;
	}

	private Document getDocument(){
		Document document = null;	// root of the xml with childs
		
        DocumentBuilderFactory factory =
            DocumentBuilderFactory.newInstance();
        try {
           DocumentBuilder builder = factory.newDocumentBuilder();
           document = builder.parse( new File(fileName) );
        } catch (SAXException sxe) {
            // Error generated during parsing)
            Exception  x = sxe;
            if (sxe.getException() != null)
                x = sxe.getException();
            x.printStackTrace();

         } catch (ParserConfigurationException pce) {
             // Parser with specified options can't be built
             pce.printStackTrace();

         } catch (IOException ioe) {
            // I/O error
            ioe.printStackTrace();
         }

         return document;

	}
	
	/**
	 * 
	 * Traverse the xml tree and inserts all mapping rules to hash table 
	 * 
	 * @return Hashtable<String, Node> containing all MappingRule nodes
	 */
	private Hashtable<String, OfferFieldMetadataObject> getMappingRuleNodes(){
		if(mappingRuleNodes == null){
			mappingRuleNodes = new Hashtable<String, OfferFieldMetadataObject>();
			Node node = getDocument().getFirstChild(); 
			for(int i = 0; i < node.getChildNodes().getLength(); i++){
				if(node.getChildNodes().item(i).getNodeName().compareTo("#text")==0) continue;
//				System.out.println("key " + node.getChildNodes().item(i).getChildNodes().item(1).getFirstChild().getNodeValue());
				mappingRuleNodes.put(node.getChildNodes().item(i).getChildNodes().item(1).getFirstChild().getNodeValue(), transformToMetadataObject(node.getChildNodes().item(i)));
			}
		}
		
		return mappingRuleNodes;
	}
	
	
	/**
	 * 
	 * P.B. Transforms Node (containing metadata) to OfferFieldMetadataObject
	 * C, 29.3.2006, P.B.: extended by reading the BeanName property from XML
	 * 
	 * @param node input to transformation
	 * @return object transformed node
	 */
	public static OfferFieldMetadataObject transformToMetadataObject(Node node){
		OfferFieldMetadataObject object = new OfferFieldMetadataObject();
		
        for(int i = 0; i < node.getChildNodes().getLength(); i++){
      	   if(node.getChildNodes().item(i).getNodeName().compareTo("#text")==0) continue;
//      	   System.out.println("property: " + node.getChildNodes().item(i).getNodeName());
//      	   System.out.println("value: " + node.getChildNodes().item(i).getFirstChild().getNodeValue());
      	   
      	   if(node.getChildNodes().item(i).getNodeName().compareTo("BeanFieldName") == 0){
      		   object.setBeanFieldName(node.getChildNodes().item(i).getFirstChild().getNodeValue());
      		   continue;
      	   }
      	   if(node.getChildNodes().item(i).getNodeName().compareTo("OntologyPropertyURI") == 0){
      		   object.setOntologyPropertyURI(node.getChildNodes().item(i).getFirstChild().getNodeValue());
      		   continue;
      	   }
      	   if(node.getChildNodes().item(i).getNodeName().compareTo("Type") == 0){
      		   object.setType(node.getChildNodes().item(i).getFirstChild().getNodeValue());
      		   continue;
      	   }
      	   if(node.getChildNodes().item(i).getNodeName().compareTo("Recursive") == 0){
      		   object.setRecursive(node.getChildNodes().item(i).getFirstChild().getNodeValue());
      		   continue;
      	   }
      	   if(node.getChildNodes().item(i).getNodeName().compareTo("RecursivelyDelete") == 0){
      		   object.setRecursivelyDelete(node.getChildNodes().item(i).getFirstChild().getNodeValue());
      		   continue;
      	   }
      	   if(node.getChildNodes().item(i).getNodeName().compareTo("ObjectNamespace") == 0){
      		   object.setObjectNamespace(node.getChildNodes().item(i).getFirstChild().getNodeValue());
      		   continue;
      	   }
      	   if(node.getChildNodes().item(i).getNodeName().compareTo("Multiple") == 0){
      		   object.setMultiple(node.getChildNodes().item(i).getFirstChild().getNodeValue());
      		   continue;
      	   }
      	   if(node.getChildNodes().item(i).getNodeName().compareTo("BeanName") == 0){
      		   object.setBeanName(node.getChildNodes().item(i).getFirstChild().getNodeValue());
      		   continue;
      	   }


      	   if(node.getChildNodes().item(i).getNodeName().compareTo("NewEntityTriple") == 0){
      		   MyStatement statement = new MyStatement();
//      		   System.out.println("subject: " + node.getChildNodes().item(i).getChildNodes().item(1).getFirstChild().getNodeValue());
//      		   System.out.println("predicate: " + node.getChildNodes().item(i).getChildNodes().item(3).getFirstChild().getNodeValue());
//      		   System.out.println("object: " + node.getChildNodes().item(i).getChildNodes().item(5).getFirstChild().getNodeValue());
//      		   System.out.println();
      		   statement.setSubject(node.getChildNodes().item(i).getChildNodes().item(1).getFirstChild().getNodeValue());
      		   statement.setPredicate(node.getChildNodes().item(i).getChildNodes().item(3).getFirstChild().getNodeValue());
      		   statement.setObject(node.getChildNodes().item(i).getChildNodes().item(5).getFirstChild().getNodeValue());
    		   object.addNewEntityTriple(statement);
    		   continue;
    	   }
      	   
         }
		
		return object;
	}
	
	/**
	 * 
	 * Search the metadata about the required field
	 * 
	 * @param fieldName the name of the field, which metadata we want
	 * @return object - instance of OfferFieldMetadataObject, containing metadata for required field or null if such field is unknown
	 */
	public OfferFieldMetadataObject getPropertyValue(String fieldName){
		
		OfferFieldMetadataObject object = getMappingRuleNodes().get(fieldName);
		
		
		return object;
	}
	
	/**
	 *  N 25.3.2006, PB
	 * 
	 * get metadata object obout field, which has such predicate
	 * 
	 * @param predicate
	 * @param fieldPrefix
	 * @return
	 */
	public OfferFieldMetadataObject getMetadataObject(String predicate, String fieldPrefix){
		OfferFieldMetadataObject metaObject = new OfferFieldMetadataObject();
		Node node = getDocument().getFirstChild(); 
		
		for(int i = 0; i < node.getChildNodes().getLength(); i++){
			if(node.getChildNodes().item(i).getNodeName().compareTo("#text")==0) continue;
//			System.out.println("key " + node.getChildNodes().item(i).getChildNodes().item(1).getFirstChild().getNodeValue());
			metaObject = transformToMetadataObject(node.getChildNodes().item(i));
//			System.out.println("m: " + metaObject.getOntologyPropertyURI());
			
			if(metaObject.getOntologyPropertyURI().compareTo(predicate) == 0){
//				System.out.println("m: " + metaObject.getBeanFieldName());
//				System.out.println("m: " + getAfterPrefix(fieldPrefix, metaObject.getBeanFieldName()));
				if(cutPrefix(fieldPrefix, metaObject.getBeanFieldName()) != null){
					if(containsTwoDots(cutPrefix(fieldPrefix, metaObject.getBeanFieldName()))){
						return metaObject;
					}
				}
			}
			
//			mappingRuleNodes.put(node.getChildNodes().item(i).getChildNodes().item(1).getFirstChild().getNodeValue(), transformToMetadataObject(node.getChildNodes().item(i)));
		}

		
		
		return null;
	}
	
	private boolean containsTwoDots(String string){
		int dotCount = 0;
		
		for(int i = 0; i < string.length(); i++){
			if(string.charAt(i) == '.'){
				dotCount++;
			}
		}

		if(dotCount > 1){
			return false;
		}
		return true;
	}

	/**
	 * N 25.3.2006, PB
	 * 
	 * @param prefix
	 * @param string
	 * @return end part of string if prefix is a maximum prefix of string, return null if string has not such prefix
	 */
	private String cutPrefix(String prefix, String string){
		String result = null;
		if(!string.startsWith(prefix)){
			return null;
		}
		result = string.substring(prefix.length(), string.length()-1);
		
		return result;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {

		GraphBeanTransformerMetadataXMLReader reader = new GraphBeanTransformerMetadataXMLReader("GraphBeanTransformerMetadata.xml");
		
		OfferFieldMetadataObject object = reader.getMetadataObject("http://www.w3.org/1999/02/22-rdf-syntax-ns#type", "");
		
		System.out.println(object.getBeanFieldName() + " " + object.getOntologyPropertyURI() + " " + object.getRecursive() + " " + object.getRecursivelyDelete() + " " + object.getType() + " " + object.getObjectNamespace());
		
		
		
//		OfferFieldMetadataObject object = reader.getPropertyValue("contractType");
//		System.out.println(object.getBeanFieldName() + " " + object.getOntologyPropertyURI() + " " + object.getRecursive() + " " + object.getRecursivelyDelete() + " " + object.getType() + " " + object.getObjectNamespace());
//		for(int i = 0; i < object.getNewEntityTriples().size(); i++){
//		   System.out.println("subject: " + object.getNewEntityTriples().get(i).getSubject());
//		   System.out.println("predicate: " + object.getNewEntityTriples().get(i).getPredicate());
//		   System.out.println("object: " + object.getNewEntityTriples().get(i).getObject());
//		   System.out.println();
//		}
		
		
		
//		String fileName = "GraphBeanTransformerMetadata.xml";
//		Document document;
//		
//        if (fileName == null) {
//            System.err.println("Usage: java DomEcho filename");
//            System.exit(1);
//        }
//
//        DocumentBuilderFactory factory =
//            DocumentBuilderFactory.newInstance();
//        //factory.setValidating(true);   
//        //factory.setNamespaceAware(true);
//        try {
//           DocumentBuilder builder = factory.newDocumentBuilder();
//           document = builder.parse( new File(fileName) );
//           
////           NodeList nodeList = document.getElementsByTagName("MappingRule");
//           org.w3c.dom.Node node =  document.getFirstChild().getChildNodes().item(3);
//           System.out.println("childs: " + node.getChildNodes().getLength()); 
////           System.out.println(document.getChildNodes().item(0).getChildNodes().getLength() + "item: " + node.getChildNodes().getLength());
//           for(int i = 0; i < node.getChildNodes().getLength(); i++){
//        	   if(node.getChildNodes().item(i).getNodeName().compareTo("#text")==0) continue;
//        	   System.out.println("item: " + node.getChildNodes().item(i).getNodeName());
//        	   System.out.println("item: " + node.getChildNodes().item(i).getFirstChild().getNodeValue());
//           }
// 
//        } catch (SAXException sxe) {
//           // Error generated during parsing)
//           Exception  x = sxe;
//           if (sxe.getException() != null)
//               x = sxe.getException();
//           x.printStackTrace();
//
//        } catch (ParserConfigurationException pce) {
//            // Parser with specified options can't be built
//            pce.printStackTrace();
//
//        } catch (IOException ioe) {
//           // I/O error
//           ioe.printStackTrace();
//        }

	}
	
}
