package nazou.support.jobofferportal.repository;


import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import javax.swing.JOptionPane;

import org.openrdf.sesame.Sesame;
import org.openrdf.sesame.config.AccessDeniedException;
import org.openrdf.sesame.config.ConfigurationException;
import org.openrdf.sesame.constants.QueryLanguage;
import org.openrdf.sesame.query.MalformedQueryException;
import org.openrdf.sesame.query.QueryEvaluationException;
import org.openrdf.sesame.repository.SesameRepository;
import org.openrdf.sesame.repository.SesameService;
import org.openrdf.sesame.sail.StatementIterator;
import org.openrdf.model.*;
import org.openrdf.model.impl.LiteralImpl;
import org.openrdf.model.impl.StatementImpl;
import org.openrdf.model.impl.URIImpl;

import nazou.support.jobofferportal.model.*;
import nazou.support.jobofferportal.runtimeutil.*;

/**
 * 
 * Implementation of interface RepositoryAccess, used to connect to Sesame repository
 * 
 * @author Peter Bartalos
 *
 */
public class SesameRepositoryAccess implements RepositoryAccess {

	private static SesameService service = null;
	private static String repositoryID = null;
	private static String userPassword = null;
	private static String userName = null; 
	private static String repositoryURL = null;
	private static SesameRepository repository = null;
	private String metadataFileName = null;
	
	public SesameRepositoryAccess(String metadataFileName) {
		super();
		setMetadataFileName(metadataFileName);
	}

	public SesameRepositoryAccess(String repositoryURL, String repositoryID, String userName, String userPassword, String metadataFileName) {
		super();
		if(repository == null){
			this.repositoryID = repositoryID;
			this.userPassword = userPassword;
			this.userName = userName;
			this.repositoryURL = repositoryURL;
			setMetadataFileName(metadataFileName);
		}
	}
	
	public SesameRepositoryAccess(String contextPath, String metadataFileName) throws IOException {
		super();
		setMetadataFileName(contextPath + "/" +  metadataFileName);
		if(repository == null){
//			getting the properties object
	        Properties properties = new Properties();
	        FileInputStream fileStream = new FileInputStream(contextPath + "/" + "RepositoryAccess.properties");
//	        FileInputStream fileStream = new FileInputStream(contextPath + "/" + metadataFileName);
	        properties.load(fileStream);

			SesameRepositoryAccess.repositoryID = properties.getProperty("repositoryID");
			SesameRepositoryAccess.userPassword = properties.getProperty("userPassword");
			SesameRepositoryAccess.userName = properties.getProperty("userName");
			SesameRepositoryAccess.repositoryURL = properties.getProperty("repositoryURL");
		}
	}

	public JobOffer downloadOffer(String offerID) throws Exception {
		
		try{
		
		BeanGraphTransformer beanToGraphTransformer= new BeanGraphTransformer(getMetadataFileName());
		JobOffer offer = new JobOffer();
		Graph graph = null;
		BeanParent bean = null;

		graph = getOfferGraphFromRepository(offerID);

		if(isEmptyGraph(graph)){
			return null;
		}
		beanToGraphTransformer.transformGraphToBean(graph, offer, "");
		return offer;
		}
		catch(Exception e){
			e.printStackTrace();
			throw e;
		}
	}
	
	
	public Organization downloadOrganization(String organizationID) throws Exception {
		
		try{
		
		BeanGraphTransformer beanToGraphTransformer= new BeanGraphTransformer(getMetadataFileName());
		Organization organization = new Organization();
		Graph graph = null;
		BeanParent bean = null;

		graph = getOfferGraphFromRepository(organizationID);

		if(isEmptyGraph(graph)){
			return null;
		}
		beanToGraphTransformer.transformGraphToBean(graph, organization, "");
		return organization;
		}
		catch(Exception e){
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * N 18.3.2006, PB
	 * C 25.3.2006, PB: added objectType case, refactoring
	 * Transforms any bean to list of graphs like its equivalent
	 * 
	 * @param bean which graph we want to delete
	 */
	public void deleteBean(BeanParent bean, String propertyPrefix){
		
		if(bean == null){
			return;
		}
		String[] fieldNames = getFieldNames(bean.getClass().getName());
		OfferFieldMetadataObject metaObject = null;
		GraphBeanTransformerMetadataXMLReader reader = new GraphBeanTransformerMetadataXMLReader(getMetadataFileName());
		Field[] fields = getFields(bean.getClass().getName());
		deleteGrafFromRepository(bean.getID());
			
		try{
			for(int i = 0; i < fields.length; i++){
				metaObject = reader.getPropertyValue(propertyPrefix + "." + fieldNames[i]);
				if(metaObject == null) continue;
				if(metaObject.getRecursivelyDelete().compareTo("true") == 0){
					if(fields[i].getType().toString().compareTo("interface java.util.List") == 0){
						List list = (List) getFieldValue(bean, fieldNames[i]);
						for(int j = 0; j < list.size(); j++){
							deleteBean((BeanParent) list.get(j), propertyPrefix + "." + fieldNames[i]);
						}
					}
					else{
						deleteBean((BeanParent) getFieldValue(bean, fieldNames[i]), propertyPrefix + "." + fieldNames[i]);						
					}


				}
			
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}

		
	}
	
	public void deleteOffer(String offerID) throws Exception{
		try{
			JobOffer offer = this.downloadOffer(offerID);
			this.deleteBean(offer, "");
		}
		catch(Exception e)
		{
			e.printStackTrace();
			throw e;
		}
	}
	
	public void deleteGrafFromRepository(String ID){
		
		try {
			if(ID == null){
				return;
			}
			// query for whole subgraph of one JobOffer
			String query = 	"CONSTRUCT * " +
							"FROM {<" + ID + ">} _ {}";
			
			// create a graph to hold the subgraph to be deleted
			Graph queryGraph = new org.openrdf.model.impl.GraphImpl();
//			System.out.println("Deleting: " + query);
			// retrieve a full subgraph
			// only performGraphQuery() returns Graph
			
			queryGraph = repository.performGraphQuery(QueryLanguage.SERQL, query);
			if(!queryGraph.getStatements().hasNext()) {
//				System.out.println("OFFER DOES NOT EXIST!");
				return;
			}
			
			repository.removeGraph(queryGraph);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	

	public Graph getOfferGraphFromRepository(String offerID){
		Graph graph= new org.openrdf.model.impl.GraphImpl();
		
			try{

				String query = 	"CONSTRUCT * " +
				"FROM {<" + offerID + ">} _ {}";

				graph = getRepository().performGraphQuery(QueryLanguage.SERQL, query);
					return graph;
				}
			catch (Exception e){
				e.printStackTrace();
			}
				
			   
		return graph;
	}	

	
	public String uploadOffer(BeanParent bean) throws Exception {
		try{
		BeanGraphTransformer beanToGraphTransformer= new BeanGraphTransformer(getMetadataFileName());
		this.setIDs(bean);
		deleteBean(bean, "");
		
		List<Graph> graphList = new ArrayList<Graph>(); 
		beanToGraphTransformer.transformBeanToGraph(bean, graphList, "");
		
		for(int i = 0; i < graphList.size(); i++){
			upload(graphList.get(i));
		}

		return bean.getID();

		}
		catch(Exception e){
			e.printStackTrace();
			throw e;
		}
		
	}
	
	
	public void connect(){
		
			
		try{
		
			if(this.getRepository() == null){
			
				java.net.URL sesameServerURL = new java.net.URL(getRepositoryURL());
		
				setService(Sesame.getService(sesameServerURL));

				getService().login(getUserName(), getUserPassword());

				setRepository(getService().getRepository(getRepositoryID()));
			}
		
		}
		catch (Exception e){
			e.printStackTrace();
		}

		
	}
	public void disconnect(){
		try{
			if(repository != null){
				getService().logout();
				repository = null;
			}
			}
			catch(Exception e){
				e.printStackTrace();
			}
	}
	
	public void upload(Object graph){
		try{

			Graph g = (Graph) graph;
			
//			StatementIterator iterator;
//			Statement statement = null;
//			iterator = g.getStatements();
//			while(iterator.hasNext()){
//				statement = iterator.next();
//				System.out.println("statement: " + statement.toString());
//			}
			
			if(!g.contains(null, null, null)){
				return;
			}


			getRepository().addGraph((Graph) graph);
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}

	public void remove(Object graph){
		try{
			getRepository().removeGraph((Graph) graph);
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public String getRepositoryID() {
		return repositoryID;
	}

	public void setRepositoryID(String repositoryID) {
		this.repositoryID = repositoryID;
	}

	public String getRepositoryURL() {
		return repositoryURL;
	}

	public void setRepositoryURL(String repositoryURL) {
		this.repositoryURL = repositoryURL;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	public String getUserPassword() {
		return userPassword;
	}

	public void setUserPassword(String userPassword) {
		this.userPassword = userPassword;
	}

	public SesameService getService() {
		return service;
	}

	public void setService(SesameService service) {
		this.service = service;
	}

	public SesameRepository getRepository() {
		return repository;
	}

	public void setRepository(SesameRepository repository) {
		this.repository = repository;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		try{
			PropertyReader propertyReader = new PropertyReader();
			propertyReader.loadProperties();
			
//			SesameRepositoryAccess repositoryAccess= new SesameRepositoryAccess(propertyReader.getRepositoryURL(), propertyReader.getRepositoryID(), propertyReader.getUserName(), propertyReader.getUserPassword(), "webapps/cocoon/portal/GraphBeanTransformerMetadata.xml");
//			SesameRepositoryAccess repositoryAccess= new SesameRepositoryAccess("./webapps" + "/cocoon" + "/portal","GraphBeanTransformerMetadata.xml");
			SesameRepositoryAccess repositoryAccess= new SesameRepositoryAccess(".","GraphBeanTransformerMetadata.xml");
			repositoryAccess.connect();

			

//			JobOffer offer = new JobOffer();
			JobOffer offer = repositoryAccess.downloadOffer("http://nazou.fiit.stuba.sk/nazou/ontologies/v0.6.16/offer-job-inst#S010_lences_01100");

			System.out.println("downloaded: " + offer.getID());

//			offer.setID("http://nazou.fiit.stuba.sk/nazou/ontologies/v0.6.16/offer-job-inst#Offer_test");
//			offer.setName("fungujeeee jak omega");
//			offer.getHasContractTypes().setText("Contract text");
			offer.setText("texticek");
			Calendar calendar = Calendar.getInstance();
			Date date = calendar.getTime();
			offer.setStartDate(date);
			
			
			System.out.println("uploaded: " + repositoryAccess.uploadOffer(offer));
			
//			repositoryAccess.deleteOffer("http://nazou.fiit.stuba.sk/nazou/ontologies/v0.6.16/offer-job-inst#S007_lekavy_01073");
			
			repositoryAccess.disconnect();		
		}
		catch(Exception e){
			e.printStackTrace();
			System.out.print("Error occured");
		}

	}
	

	 /**
	 * Sets the bean IDs if they are not set
	 * C 19.3.2006, PB: set ID recursively for all subbean
	 * 
	 * @param bean the bean whos IDs (offer ID, salary ID, offered by organization ID) we want to set
	 * @return Offer bean with IDs set
	 */
	public void setIDs(BeanParent bean){
		ID id= new ID();
		IDLists idLists= new IDLists(getRepository());
		
		Field[] fields = getFields(bean.getClass().getName());
		String[] fieldNames = getFieldNames(bean.getClass().getName());

		try{
			if(bean.getID() == null){
				bean.setID(id.getID("http://nazou.fiit.stuba.sk/nazou/ontologies/v0.6.16/offer-job-inst#" + getAfterLastDotSubstring(bean.getClass().getName())));
			}
			
			for(int i = 0; i < fields.length; i++){
				if(fields[i].getType().toString().compareTo("class java.util.Date") == 0) continue;
				if(fields[i].getType().toString().compareTo("class java.lang.Float") == 0) continue;
				if(fields[i].getType().toString().compareTo("class java.lang.Integer") == 0) continue;
				if(fields[i].getType().toString().compareTo("class java.lang.Boolean") == 0) continue;
				if(fields[i].getType().toString().compareTo("class java.lang.String") != 0){
					if(fields[i].getType().toString().compareTo("interface java.util.List") == 0){
						List list = (List) getFieldValue(bean, fieldNames[i]);
						for(int j = 0; j < list.size(); j++){
							String s = "";
							if(s.getClass().isInstance(list.get(j))){
								break;
							}
							setIDs((BeanParent) list.get(j));
						}
					}
					else if(getFieldValue(bean, fieldNames[i]) != null){
							setIDs((BeanParent) getFieldValue(bean, fieldNames[i]));
						 }
					
				}
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
	

	}
	
	
	
	
	
	 /**
	 * Sets the bean attributes (for testing purposes). Goes recursively for subbeans until deep is > 0
	 * N 29.3.2006, PB
	 * 
	 * @param bean the bean whos attributes we want to set
	 * @return
	 */
	public BeanParent setBeanAttributes(BeanParent bean, String propertyPrefix, int deep){
		ID id= new ID();
		IDLists idLists= new IDLists(getRepository());
		GraphBeanTransformerMetadataXMLReader reader = new GraphBeanTransformerMetadataXMLReader(getMetadataFileName());
		Field[] fields = getFields(bean.getClass().getName());
		String value = null;
		
		if(deep == 0){
			return null;
		}
		
		try{
			if(bean.getID() == null){
			bean.setID(id.getID("http://nazou.fiit.stuba.sk/nazou/ontologies/v0.6.16/offer-job-inst#" + getAfterLastDotSubstring(bean.getClass().getName())));
		}

			for(int i = 0; i < fields.length; i++){
				if(fields[i].getType().toString().compareTo("class java.lang.String") == 0){
					value = "test_" + fields[i].getName().toString() + i;
//					value = "" + i;

					OfferFieldMetadataObject metaObject = reader.getPropertyValue(propertyPrefix + "." + fields[i].getName());
					if(metaObject == null) continue;
					String type = metaObject.getType();
					if(type == null) continue;
					if(type.compareTo("int") == 0){
						value = "" + i;
					}
					else if(type.compareTo("float") == 0){
						value = "" + i + ".0";
					}
					else if(type.compareTo("boolean") == 0){
						value = "true";
					}
					if(type.compareTo("objectType") == 0){
						value = "http://nazou.fiit.stuba.sk/nazou/ontologies/v0.6.16/offer-job-inst#" + value;
					}
					setField(bean, fields[i].getName().toString(), value);
				}
				else if(fields[i].getType().toString().compareTo("interface java.util.List") == 0){
					List list = (List) getFieldValue(bean, fields[i].getName());
					for(int j = 0; j < 2; j++){
						OfferFieldMetadataObject metaObject = reader.getPropertyValue(propertyPrefix + "." + fields[i].getName());
						if(metaObject == null){
							continue;
						}
						if(metaObject.getType().compareTo("object") != 0 && metaObject.getType().compareTo("objectType") != 0){
							list.add("" + j);
						}
						else if(metaObject.getType().compareTo("objectType") == 0){
							String className = metaObject.getBeanName();
							list.add("http://nazou.fiit.stuba.sk/nazou/ontologies/v0.6.16/offer-job-inst#" + className + j);
						}
						else{
							String className = metaObject.getBeanName();
							if(className == null){
								continue;
							}
							Class c = Class.forName("nazou.support.jobofferportal.model." + className);

							BeanParent subBean = (BeanParent) c.newInstance();
							list.add(subBean);
							setBeanAttributes(subBean, propertyPrefix + "." + fields[i].getName(), deep - 1);
						}
					}

				}
				else if(fields[i].getType().toString().compareTo("class java.util.Date") == 0){
					Calendar calendar = Calendar.getInstance();
					Date date = calendar.getTime();
					setField(bean, fields[i].getName().toString(), date);
				}
				else{
					BeanParent subBean = (BeanParent) fields[i].getType().newInstance();
					setField(bean, fields[i].getName().toString(), subBean);
					setBeanAttributes(subBean, propertyPrefix + "." + fields[i].getName(), deep - 1);
				}

			}
			

		}
		catch(Exception e){
			e.printStackTrace();
		}
		
		
		return bean;
	}
	
	/**
	 * N 27.3.2006, PB
	 * 
	 * Sets the field fieldName in a object to the value
	 * 
	 * @param object
	 * @param fieldName
	 * @param value
	 */
	private void setField(Object object, String fieldName, Object value){
		try{

			Class c = object.getClass();
			Method m = c.getMethod( toSetterName(fieldName), new Class[] {value.getClass()} );
			m.invoke( object, new Object[] {value} );
			
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private Object getFieldValue(Object object, String fieldName){
		Object value = null;
		try{
			Class c = object.getClass();
			Method m = c.getMethod( toGetterName(fieldName), null );
			value = m.invoke( object, null );
//			System.out.println("method: " + toGetterName(fieldName) + " value " + value);
			return value;
			
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return value;
	}
	
	private String toSetterName(String name)
	{
		StringBuffer sb = new StringBuffer();
		sb.append( "set" ).append( name.substring(0,1).toUpperCase() ).append( name.substring(1) );
		return sb.toString();
	}
	
	private String toGetterName(String name)
	{
		StringBuffer sb = new StringBuffer();
		sb.append( "get" ).append( name.substring(0,1).toUpperCase() ).append( name.substring(1) );
		return sb.toString();
	}
	
	private String getAfterLastDotSubstring(String string){
		String s = string.replace('.', '-');
		String[] str = s.split("-"); 
		
		return str[str.length - 1];
	}
	
	private Field[] getFields(String className) {
		Field fields[] = null;
		try{
			Class c = Class.forName(className);
			fields = c.getDeclaredFields();
		}
		catch(ClassNotFoundException e){
			e.printStackTrace();
		}
		return fields;
	}
	
	private boolean isEmptyGraph(Graph graph){
		StatementIterator iterator;
		iterator= graph.getStatements(null, null, null);
		if(iterator.hasNext()){
			return false;
		}
		return true;
	}
	
	protected String cutNamespace(String statement){
		String[] s= statement.split("#");
		return s[1];
	}
	
	private String[] getFieldNames(String className) {
		String[] fieldNames = null;
		try{
			Class c = Class.forName(className);
			Field fields[] = c.getDeclaredFields();
			fieldNames = new String[fields.length];
			
			for(int i = 0; i < fieldNames.length; i++){
				String str = fields[i].toString().replace('.', '-');
				String[] s = str.split("-");
				
				fieldNames[i] = s[s.length - 1];
			}
			
		}
		catch(ClassNotFoundException e){
			e.printStackTrace();
		}
		return fieldNames;
	}

	public String getMetadataFileName() {
		return metadataFileName;
	}

	public void setMetadataFileName(String metadataFileName) {
		this.metadataFileName = metadataFileName;
	}
}
