package nazou.support.jobofferportal.repository;

import org.openrdf.model.Literal;
import org.openrdf.model.URI;
import org.openrdf.model.ValueFactory;
import org.openrdf.model.Graph;
import org.openrdf.model.Statement;
import org.openrdf.model.impl.*;

/**
 * 
 * Creation of statements for RDF graph
 * 
 * @author Peter Bartalos
 *
 */
public class StatementGenerator {

	Namespaces namespace;
	protected ValueFactory myFactory;
	Graph graph;
	
	public StatementGenerator() {
		super();
		namespace= new Namespaces();
		graph= new org.openrdf.model.impl.GraphImpl();
		myFactory= graph.getValueFactory();
	}

	public Statement getOfferIDStatement(String offerID){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceRdf(), "type");
		URI myObject = myFactory.createURI(namespace.getNamespaceJo(), "JobOffer");
		return new StatementImpl(mySubject, myPredicate,  myObject);
	}	

	public Statement getOfferTypeStatement(String offerID){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceRdf(), "type");
		URI myObject = myFactory.createURI(namespace.getNamespaceJo(), "JobOffer");
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
	
	public Statement getOfferNameStatement(String offerID, String _name){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceOfr(), "name");
//		Literal myObject = myFactory.createLiteral(_name);
		Literal myObject = myFactory.createLiteral(_name, new URIImpl("http://www.w3.org/2001/XMLSchema#string"));
		
		
		return new StatementImpl(mySubject, myPredicate, myObject);
	}
	
	public Statement getOfferTextStatement(String offerID, String _text){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "text");
		Literal myObject = myFactory.createLiteral(_text);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getOfferHoursPerWeekStatement(String offerID, String hoursPerWeek){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "hoursPerWeek");
		Literal myObject = myFactory.createLiteral(hoursPerWeek);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
	
	public Statement getOfferSummaryStatement(String offerID, String summary){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "summary");
		Literal myObject = myFactory.createLiteral(summary);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
	

	public Statement getOfferHasBenefitStatement(String offerID, String benefitTyp){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "hasBenefit");
		URI myObject = myFactory.createURI(namespace.getNamespaceOfferInst(), benefitTyp);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getBenefitTypStatement(String benefitID, String benefitTyp){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), benefitID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceRdf(), "type");
		URI myObject = myFactory.createURI(namespace.getNamespaceJo(), benefitTyp);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
	
	public Statement getBenefitStatement(String benefitID, String benefit){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), benefitID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "text");
		Literal myObject = myFactory.createLiteral(benefit);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
	
	public Statement getOfferIsOfferedViaStatement(String offerID, String organizationID){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "isOfferedVia");
		URI myObject = myFactory.createURI(namespace.getNamespaceOfferInst(), organizationID);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getOfferIsOfferedByStatement(String offerID, String organizationID){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "isOfferedBy");
		URI myObject = myFactory.createURI(namespace.getNamespaceOfferInst(), organizationID);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getOfferHasSalaryStatement(String offerID, String salaryID){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "hasSalary");
		URI myObject = myFactory.createURI(namespace.getNamespaceOfferInst(), salaryID);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getSalaryTypeStatement(String salaryID){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), salaryID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceRdf(), "type");
		URI myObject = myFactory.createURI(namespace.getNamespaceJo(), "Salary");
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
	
	public Statement getSalaryTextStatement(String salaryID, String text){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), salaryID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "text");
		Literal myObject = myFactory.createLiteral(text);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getSalaryMinAmountStatement(String salaryID, String minAmount){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), salaryID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "minAmount");
		Literal myObject = myFactory.createLiteral(minAmount);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
	
	public Statement getSalaryMaxAmountStatement(String salaryID, String maxAmount){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), salaryID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "maxAmount");
		Literal myObject = myFactory.createLiteral(maxAmount);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getSalaryBonusStatement(String salaryID, String bonus){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), salaryID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "bonus");
		Literal myObject = myFactory.createLiteral(bonus);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
	
	public Statement getOrganizationNameStatement(String organizationID, String name){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), organizationID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "name");
		Literal myObject = myFactory.createLiteral(name);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getOrganizationTextStatement(String organizationID, String text){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), organizationID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "text");
		Literal myObject = myFactory.createLiteral(text);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getOrganizationProfileStatement(String organizationID, String profile){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), organizationID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "profile");
		Literal myObject = myFactory.createLiteral(profile);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
	
	public Statement getOrganizationTypeStatement(String organizationID){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), organizationID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceRdf(), "type");
		URI myObject = myFactory.createURI(namespace.getNamespaceJo(), "Organization");
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getOrganizationOffersStatement(String organizationID, String offerID){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), organizationID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "offers");
		URI myObject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}

	public Statement getOrganizationMediatesStatement(String organizationID, String offerID){
		URI mySubject = myFactory.createURI(namespace.getNamespaceOfferInst(), organizationID);
		URI myPredicate = myFactory.createURI(namespace.getNamespaceJo(), "mediates");
		URI myObject = myFactory.createURI(namespace.getNamespaceOfferInst(), offerID);
		
		return new StatementImpl(mySubject, myPredicate, myObject);		
	}
}
